package JElriDEV;

import java.awt.*;
import java.awt.event.*;
import java.applet.*;
import java.net.*;
import java.util.*;

/*
 * (C) ELRIDEV SOFTWARE, 1999. ALL RIGHTS RESERVED.
 * THIS SOFTWARE IS THE PROPERTY OF ELRIDEV SOFTWARE, YOU CAN USE THIS SOFTWARE
 * ON YOUR WEBPAGE (OR USE THE SOURCE FOR LEARNING JAVA), IT PROVIDED AS AN EXAMPLE,
 * WITH NO WARRANTIES WHATSOEVER, SHOULD ANY DAMAGE OCCUR FROM THE USE OF THIS
 * PRODUCT ELRIDEV WILL BE HELD IN NO WAY LIABLE FOR DAMAGES.
 * YOU CAN ALSO MODIFY THIS CODE FOR YOU OWN PURPOSES, HOWEVER ONCE THIS IS DONE
 * YOU MUST INDICATE THAT THESE CHANGES HAVE BEEN MADE, SHOULD YOU WISH IT REMAIN
 * FREELY AVAILABLE.
 */

/**
 * spinner applet, this applet is used to produce a spinning vector image. to
 * design the image on should the Spinner Visual C++ program.
 */
public class Spinner extends Applet implements Runnable{
    boolean isStandalone = false;
    // the filename to load
    String spinFile;
    // the rotation and scale speeds
    int xaxisRotation;
    int ylenRotation;
    int xlenRotation;
    int scaleSpeed;
    boolean bShading=false;

    // the current rotation in degrees
    private int rotation = 0;
    // if the scale is growing or shrinking
	private boolean growing=true;
    // present scale of image
	private double scale = 0.00;
    // length adjustments
    private double act_lenx=1;
    private double act_leny=1;
    // scale min and max values
    private double scaleMin,scaleMax;

    Image imgBuffer; // for double buffering

    protected Vector shapes; // list of loaded shapes from file

    /**
     * read a parameter but has a default value
     * @param key the key value
     * @param def the default to return if param doesnt exist
     */
    public String getParameter(String key, String def)
    {
        return isStandalone ? System.getProperty(key, def) :
               (getParameter(key) != null ? getParameter(key) : def);
    }

    /**
     * create the applet on its own thread.
     */
    public Spinner()
    {
        super();
        Thread th = new Thread(this);
        th.start();
    }

    /**
     * redraw the screen and change all the rotation values on by one
     */
    public void refreshScreen()
    {
        if(imgBuffer==null) return;
        Graphics gBuffer= imgBuffer.getGraphics();
		Dimension sizeWnd = size();
		Point centre = new Point(sizeWnd.width/2,sizeWnd.height/2);

        // add the xaxis rotation to the value.
		rotation+=(xaxisRotation/2);
		if(rotation>360) rotation = 0;
		double act_rot = (rotation/180.) * Math.PI;

        // add the xlen value
        act_lenx = act_lenx += (xlenRotation/2.);
        if(act_lenx > 360) act_lenx=0;
        act_leny = act_leny += (ylenRotation/2.);
        if(act_leny > 360) act_leny=0;
		double act_rotx = (act_lenx/180.) * Math.PI;
		double act_roty = (act_leny/180.) * Math.PI;

		// very simple inertia effect on the scale. the factor gives us a
        // simple value by which to give speed settings
        double factor = scaleSpeed/10.;
        double scale1=scaleMin;
        if(scaleMin!=scaleMax)
        {
          double sclRad = (scale/180)*Math.PI;
          scale1 = (Math.sin(sclRad)*(scaleMax-scaleMin))+scaleMin;
          if(scale>359) scale = 0;
          scale += factor;
        }
        gBuffer.setColor(getBackground());
        gBuffer.fillRect(0,0,sizeWnd.width,sizeWnd.height);
        for(int i=0;i<shapes.size();i++)
        {
            DrawableShape s = (DrawableShape) shapes.elementAt(i);
    		s.Draw(gBuffer,scale1,centre,act_rot,act_rotx,act_roty);
        }
        gBuffer.dispose();
    }

    /**
     * just redraw the new image - work done in refreshSCreen()
     * @param g graphics device
     */
    public void paint(Graphics g)
    {
        if(imgBuffer!=null) g.drawImage(imgBuffer,0,0,this);
    }

    /**
     * update, overloaded to call paint
     * @param g graphics device
     */
    public void update(Graphics g)
    {
        paint(g);
    }

    /**
     * just keeps calling refresh and repaint
     */
    public void run()
    {
        try
        {
            while(true)
            {
                Thread.sleep(40);
                refreshScreen();
                repaint();
            }
        }
        catch(InterruptedException e)
        {
        }
    }

    /**
     * Initialize the applet
     */
    public void init()
    {
        try
        {
            spinFile = this.getParameter("VectorFile", "spinfile.jvf");
            xaxisRotation = Integer.parseInt(this.getParameter("XRotation", "10"));
            scaleSpeed = Integer.parseInt(this.getParameter("ScaleSpeed", "5"));
            scaleMin = Double.valueOf(this.getParameter("ScaleMin", "2.5")).doubleValue();
            scaleMax = Double.valueOf(this.getParameter("ScaleMax", "1")).doubleValue();
            ylenRotation = Integer.parseInt(this.getParameter("YLRotation", "1"));
            xlenRotation = Integer.parseInt(this.getParameter("XLRotation", "2"));
            bShading = Boolean.valueOf(this.getParameter("LightShading", "false")).booleanValue();
            jbInit();
        }
        catch(Exception e)
        {
            showStatus("The Spinner Applet did not initialise");
            e.printStackTrace();
        }

        readInputFile(spinFile);
        Dimension d = getSize();
        imgBuffer = createImage(d.width,d.height);

        // all the speed parameters must be more than 0 but less than 10.
        scaleSpeed=Math.min(Math.max(scaleSpeed,0),10);
        xaxisRotation=Math.min(Math.max(xaxisRotation,0),10);
        ylenRotation=Math.min(Math.max(ylenRotation,0),10);
        xlenRotation=Math.min(Math.max(xlenRotation,0),10);
        scale = scaleMin;
    }

    //Component initialization
    private void jbInit() throws Exception
    {
        this.setSize(new Dimension(400,300));
    }

    /**
     * reads the input file from disk
     * @param fileToRead the file to load
     */
    protected void readInputFile(String fileToRead)
    {
        try
        {
            // open the file using the name passed in
            URL url = new URL(getDocumentBase(),fileToRead);
            URLConnection urlc = url.openConnection();
            BinaryFileReader file = new BinaryFileReader(urlc.getInputStream());

            int imagic=file.readInt();
            if(imagic!=0xfade0ff)
            {
                showStatus("File "+fileToRead+" is not a valid format");
                return;
            }

            // get the count, and bk colour information
            int  count = file.readShort();
            byte red = file.readByte();
            byte green = file.readByte();
            byte blue = file.readByte();
            Color clrBack = new Color(file.intBits(red,16)|file.intBits(green,8)|file.intBits(blue,0));
            setBackground(clrBack);

            // read the padding byte.
            file.readByte();

            shapes = new Vector(count+2);

            for(int i=0;i<count;i++)
            {
                // check if the items filled, and find how many points
                boolean fill = file.readByte()==1;
                int   items = file.readByte();

                // get the colour information
                red = file.readByte();
                green = file.readByte();
                blue = file.readByte();
                Color clrBg = new Color(file.intBits(red,16)|file.intBits(green,8)|file.intBits(blue,0));
                red = file.readByte();
                green = file.readByte();
                blue = file.readByte();
                Color clrFg = new Color(file.intBits(red,16)|file.intBits(green,8)|file.intBits(blue,0));
                // now make the drawable shape object
                DrawableShape shape = new DrawableShape(fill,clrFg,clrBg,items,bShading);
                for(int ppts=0;ppts<items;ppts++)
                {
                    shape.setPoints(ppts,file.readDouble(),file.readDouble());
                }
                shapes.addElement(shape);
                if(!file.moreAvailable()) break;
            }
        }
        catch(Exception e)
        {
            System.out.println("error reading input file:" +fileToRead);
            e.printStackTrace();
        }
    }

}
